package p2.tictactoe;
import java.io.PrintStream;
import junit.framework.TestCase;

/**
 * Test cases for TicTacToe.
 * @author $Author: oscar $
 * @version $Id: TicTacToeTest.java,v 1.4 2005/02/21 21:12:23 oscar Exp $
 */
public class TicTacToeTest extends TestCase {
	private TicTacToe game;
	
	public static void main(String[] args) {
		junit.awtui.TestRunner.run(TicTacToeTest.class);
	}
	
	/**
	 * Test the getters and setters.
	 */
	public void testState() {
		game = new TicTacToe(new Player('X'), new Player('O'));
		assertTrue(game.get(0,0) == ' ');
		assertTrue(game.get(2,2) == ' ');
		game.set(2,2,'X');
		assertTrue(game.get(2,2) == 'X');
		game.set(2,2,' ');
		assertTrue(game.get(2,2) == ' ');
		assertTrue(!game.inRange(3,3));
	}
	
	public void testXWinDiagonal() {
		checkGame("a1\nb2\nc3\n", "b1\nc1\n", "X", 4);
	}

	public void testNoWinner() {
		checkGame("b2\na1\nb3\na2\nc1\n",
				"a3\nc3\nb1\nc2\n", "nobody",0);
	}
	
	public void testOWinReverseDiagonal() {
		checkGame("a1\nb1\n"
				+ "a1\nb1\nb2\nc1\nd0\nxxx\n\n" // invalid moves
				+ "a2\n",
				"b2\nc1\na3\n", "O", 3);
	}
	
	public void testXWinCentreColumn() {
		checkGame("b2\nb1\nb3\n",
				"a1\n"
				+ "b1\nA0\n" // invalid moves
				+ "a3\n", "X", 4);
	}
	
	public void testOWinTopRow() {
		checkGame("b2\na1\nc1\n",
				"a3\nc3\nb3\n", "O", 3);
	}
	
	/**
	 * Run a game with simulated input for X and Y, and check that
	 * the winner is as expected.
	 * Game results are sent to a NullOutputStream.
	 */
	public void checkGame(String Xmoves, String Omoves, String winner, int squaresLeft) {
		Player X = new Player('X', Xmoves);
		Player O = new Player('O', Omoves);
		TicTacToe game = new TicTacToe(X, O);
		GameDriver.playGame(game, new PrintStream(new NullOutputStream()));
		assertTrue(game.winner().name().equals(winner));
		assertTrue(game.squaresLeft() == squaresLeft);
	}
}
