package p2.tictactoe;

/**
 * Helper class to find winning scores for BoardGame.
 * @author $Author: oscar $
 * @version $Id$
 */
public class Runner {

	BoardGame _game;
	// Home col and row:
	int _homeCol;
	int _homeRow;
	// Current col & row:
	int _col=0;
	int _row=0;

	/**
	 * This class does not have any interesting
	 * contracts.  We do not bother to define
	 * a separate test() method, since it will
	 * be thoroughly tested by the unit tests
	 * for the BoardGame implementations.
	 */
	public Runner(BoardGame game, int col, int row)
	{
		_game = game;
		_homeCol = col;
		_homeRow = row;
	}

	/**
	 * A Runner runs in some direction (dcol,drow)
	 * as far as it can, as long as the pieces on the
	 * Board are the same as the home square.
	 * Then it runs in the opposite direction, and
	 * returns the total score.
	 */
	public int run(int dcol, int drow)
	{
		int score = 1;
		this.goHome();
		score += this.forwardRun(dcol, drow);
		this.goHome();
		dcol = -dcol;
		drow = -drow;
		score += this.forwardRun(dcol, drow);
		return score;
	}
	
	private void goHome() {
		_col= _homeCol;
		_row = _homeRow;
	}
	
	/**
	 * Recursively move forward as long as we are
	 * in a run.  Return the length of the run.
	 */
	private int forwardRun(int dcol, int drow)
	{
		this.move(dcol, drow);
		if (this.samePlayer())
			return 1 + this.forwardRun(dcol, drow);
		else
			return 0;
	}
	
	private void move(int dcol, int drow) {
		_col = _col + dcol;
		_row = _row + drow;
	}
	
	/**
	 * The player at the current location is the
	 * same as that of our home square.
	 */
	private boolean samePlayer()
	{
		if (_game.inRange(_col,_row)) {
			char home = _game.get(_homeCol, _homeRow);
			char here = _game.get(_col, _row);
			return home == here;
		} else {
			// Oops -- we walked off the edge of the board!
			return false;
		}
	}
}
