package p2.tictactoe;

import java.io.PrintStream;
import junit.framework.TestCase;

/**
 * @author $Author: oscar $
 * @version $Id: AbstractBoardGameTest.java,v 1.2 2005/02/23 22:11:20 oscar Exp $
 */
public abstract class AbstractBoardGameTest extends TestCase {
	protected PrintStream out_, err_;
	protected BoardGame game;
	
	public AbstractBoardGameTest(String name) {
		super(name);
		silent();
		// verbose(); // useful when debugging test cases
	}
	
	protected void silent() {
		out_ = err_ = new PrintStream(new NullOutputStream());
	}

	protected void verbose() {
		out_ = System.out;
		err_ = System.err;
	}
	
	/**
	 * Run a game with simulated input for X and Y, and check that
	 * the winner is as expected.
	 * Game results are sent to a NullOutputStream.
	 */
	public void checkGame(String Xmoves, String Omoves, String winner, int squaresLeft) {
		Player X = new StreamPlayer('X', Xmoves);
		Player O = new StreamPlayer('O', Omoves);
		game = makeGame(X,O);
		GameDriver.playGame(game, out_, err_);
		assertEquals(game.winner().toString(), winner);
		assertEquals(game.squaresLeft(), squaresLeft);
	}
	
	/**
	 * Factory method so subclasses can create other games.
	 */
	abstract protected BoardGame makeGame(Player X, Player O);

	/**
	 * Run a command that should fail.
	 */
	public void assertFails(Runnable command) {
		boolean caught = false;
		try {
			command.run();
		} catch(AssertionError err) {
			caught = true;
		}
		assertTrue(caught);
	}
}
