package p2.tictactoe;

/**
 * Manage interaction with user.
 * This class implements "nobody" -- a Player that has an identity,
 * but does not make moves.  Serves as a superclass for other implementations.
 * @author $Author: oscar $
 * @version $Id: InactivePlayer.java,v 1.1 2005/02/23 22:11:19 oscar Exp $
 */
public class InactivePlayer implements Player {
	private final char mark_;
	protected BoardGame game_;

	/**
	 * The normal contructor to use.
	 * @param mark the character representing this player ('X' or 'O')
	 */
	public InactivePlayer(char mark) {
		mark_ = mark;
	}
	
	/**
	 * Special constructor for the Player representing nobody.
	 * (Used by TicTacToe as a placeholder when there is no
	 * winner yet.)
	 * @see AbstractBoardGame
	 */
	public InactivePlayer() {
		this(' ');
	}

	/**
	 * @return the char representation of this Player
	 * @see AbstractBoardGame#toString
	 */
	public char mark() { return mark_; }
	
	/**
	 * @return the String representation of this Player
	 * @see GameDriver#playGame
	 */
	public String toString() {
		if (this.isNobody())
			return "nobody";
		else
			return new Character(this.mark()).toString();
	}
	
	/**
	 * @return whether this Player represents "nobody".
	 * (By convention, a Player with a blank as its mark.)
	 */
	public boolean isNobody() {
		return this.mark() == ' ';
	}

	/**
	 * Let this player join a particular game.
	 * Set by a BoardGame when it is instantiated with Players.
	 */
	public void setGame(BoardGame game) {
		game_ = game;
	}
}
