package p2.tictactoe.gui;

import java.awt.*;

import p2.tictactoe.Player;

/**
 * Represents the view of a single place on the BoardGame.
 * An instance of PlaceListener forwards mouse clicks to a Player.
 * Here we assume that Players have the mark 'X' or 'O'.
 *
 * @author $Author: oscar $
 * @version $Id: Place.java,v 1.1 2005/02/23 22:11:21 oscar Exp $
 */
public class Place extends Panel {
	private final int col_;
	private final int row_;
	
	private final String imageDir = "p2/tictactoe/gui/images";
	private Image image_;
	private final Image oImage;
	private final Image xImage;
	private final Image blankImage;
	
	/**
	 * A Place knows its row and column number.
	 * It also knows how to display either an X or O image
	 * when its state changed.
	 */
	Place(int col, int row) {
		col_ = col;
		row_ = row;
		this.xImage = getImage(imageDir + "/cross.gif");
		this.oImage = getImage(imageDir + "/not.gif");
		this.blankImage = getImage(imageDir + "/blank.gif");
		this.clearImage();
	}
	
	private Image getImage(String imageFile){
		Image image = Toolkit.getDefaultToolkit().createImage(imageFile);
		assert image != null;
		return image;
	}
	
	/**
	 * Draws a box along its edge, and, if an image is defined,
	 * draws that too (inset 10% from the border).
	 */
	public void paint(Graphics g) {
		super.paint(g);
		Rectangle rect = g.getClipBounds();
		int h = rect.height;
		int w = rect.width;
		int offset = w/10;
		g.drawRect(0,0,w,h);
		if (image_ != null) {
			g.drawImage(image_, offset, offset, w-2*offset, h-2*offset, this);
		}
	}
	
	/**
	 * Called by PlaceListener to determine in which location
	 * the mouse has been clicked.
	 * @see PlaceListener
	 */
	public int getRow() { return row_; }
	public int getCol() { return col_; }
	
	/**
	 * Called when the BoardGame changes state.
	 * Note that we hard-wire the interpretation of the mark
	 * as either 'X' or 'O'.  This would have to be changed
	 * if the application were extended to deal with other
	 * kinds of Players.
	 */
	public void setMove(Player player) {
		if (player.mark() == 'X') {
			image_ = xImage;
		} else {
			image_ = oImage;
		}
		repaint();
	}
	
	public void clearImage() {
		setBackground(Color.white);
		image_ = blankImage;
		repaint();
	}
}
