package p2.tictactoe;

import java.util.Observer;

/**
 * Interface for TicTacToe and Gomoku.
 * @author $Author: oscar $
 * @version $Id$
 */
public interface BoardGame {
	/**
	 * Number of columns in this game.
	 */
	public int get_cols();
	/**
	 * Number of rows in this game.
	 */
	public int get_rows();
	/**
	 * Join this game to play as the returned Player.
	 * Must be called twice.
	 */
	public Player join();
	/**
	 * Two Players have joined.
	 */
	public boolean ready();
	/**
	 * Returns which Player must play now.
	 */
	public Player currentPlayer();
	/**
	 * Player p attempts to play at position (col,row).
	 */
	public void move(int col, int row, Player p)
		throws InvalidMoveException;
	/**
	 * Returns the winner of the game.
	 */
	public Player winner();
	/**
	 * Returns the Player who has played at the given position.
	 */
	public Player get(int col, int row);
	/**
	 * The game is not over.
	 * (There is no winner and there are still empty squares.)
	 */
	public boolean notOver();
	/**
	 * The number of squares left unplayed.
	 */
	public int squaresLeft();
	/**
	 * The given position is valid.
	 */
	public boolean inRange(int col, int row);
	/**
	 * Add an Observer to be notified of updates.
	 */
	public void addObserver(Observer o);
	/**
	 * Restart the game, initializing the state.
	 */
	public void restart();
	/**
	 * Return the name of the game.
	 */
	public String name();
}

