package p2.tictactoe.gui;

import java.awt.event.*;
import p2.tictactoe.BoardGame;
import p2.tictactoe.InvalidMoveException;

/**
 * This class picks up mouse clicks and propogates them
 * to a Player of the BoardGame.
 *
 * @author $Author: oscar $
 * @version $Id: PlaceListener.java,v 1.3 2005/03/02 14:32:34 oscar Exp $
 */
public class PlaceListener extends MouseAdapter {
	private final Place place_;
	private final GameGUI gui_;
	
	public PlaceListener(Place place, GameGUI gui) {
		place_ = place;
		gui_ = gui;
	}

	/**
	 * When the mouse is clicked, we ask the current player to
	 * make the corresponding move.  Report when the game is over.
	 */
	public void mouseClicked(MouseEvent e) {
		BoardGame game = gui_.getGame();
		int col = place_.getCol();
		int row = place_.getRow();
		if (game.notOver()) {
			try {
				// ((GUIplayer) game.currentPlayer()).move(col,row);
				((GUIplayer) gui_.getPlayer()).move(col,row);
				gui_.showWhoPlays();
			} catch (InvalidMoveException err) {
				gui_.showFeedBack(err.getMessage());
			}
			if (!game.notOver()) {
				gui_.showFeedBack("Game over -- " + game.winner() + " wins!");
			}
		} else {
			gui_.showFeedBack("The game is over!");
		}
	}
}

