package p2.tictactoe;

import java.io.PrintStream;
import junit.framework.TestCase;

/**
 * Abstract superclass providing common methods for TicTacToeTest and GomokuTest
 * @author $Author: oscar $
 * @version $Id: AbstractBoardGameTest.java,v 1.4 2005/03/10 15:28:25 oscar Exp $
 */
public abstract class AbstractBoardGameTest extends TestCase {
	protected PrintStream out_, err_;
	protected BoardGame game;
	
	public AbstractBoardGameTest(String name) {
		super(name);
		silent();
		// verbose(); // useful when debugging test cases
	}
	
	/**
	 * Test silently.
	 */
	protected void silent() {
		out_ = err_ = new PrintStream(new NullOutputStream());
	}

	/**
	 * Print the game state as scripted moves are played.
	 * Useful for debugging.
	 */
	protected void verbose() {
		out_ = System.out;
		err_ = System.err;
	}
	
	/**
	 * Run a game with scripted input for X and Y, and check that
	 * the winner is as expected.
	 */
	public void checkGame(String Xmoves, String Omoves, String winner, int squaresLeft) {
		Player X = new StreamPlayer('X', Xmoves);
		Player O = new StreamPlayer('O', Omoves);
		game = makeGame(X,O);
		GameDriver.playGame(game, out_, err_);
		assertEquals(game.winner().toString(), winner);
		assertEquals(game.squaresLeft(), squaresLeft);
	}
	
	/**
	 * Factory method so subclasses can create different kinds of games.
	 */
	abstract protected BoardGame makeGame(Player X, Player O);
}
