package p2.tictactoe.gui;

import java.awt.event.*;
import java.rmi.RemoteException;

import p2.tictactoe.Move;
import p2.tictactoe.rmi.RemoteGame;

/**
 * This class picks up mouse clicks and propogates them
 * to a Player of the BoardGame.
 *
 * @author $Author: oscar $
 * @version $Id: PlaceListener.java,v 1.4 2005/03/03 14:18:13 oscar Exp $
 */
public class PlaceListener extends MouseAdapter {
	private final Place place_;
	private final GameGUI gui_;
	
	public PlaceListener(Place place, GameGUI gui) {
		place_ = place;
		gui_ = gui;
	}

	/**
	 * When the mouse is clicked, forward a Move request to the
	 * RemoteGame.  We also report when the game is over.
	 **/
	public void mouseClicked(MouseEvent e) {
		RemoteGame game = gui_.getGame();
		int col = place_.getCol();
		int row = place_.getRow();
		message("Got a mouse click");
		try {
			if (game.ready()) {
				if (game.notOver()) {
					Move move = new Move(col,row, gui_.getMark());
					// Provide some feedback, in case the server is slow
					message("Attempting " + move);
					
					if (!game.move(move))
						message("Invalid move! -- "
							+ game.currentPlayer() + " plays");
					// else game will call us back with an update()
				} else {
					message("The game is over!");
				}
			} else {
				// We are missing a way to inform the user that
				// another player has connected!
				message("Please wait for a second player!");
			}
		} catch (RemoteException err) {
			// The server has crashed?
			message("Can't invoke remote game methods!" + err);
		}
	}

	private void message(String s) {
		gui_.showFeedBack(s);
	}
}

