package p2.tictactoe.server;

import java.rmi.*;
import java.rmi.server.*;

import p2.tictactoe.Move;
import p2.tictactoe.gui.GameGUI;
import p2.tictactoe.rmi.*;

/**
 * Called by the RemoteGame when it changes state.
 * This is the only Remote object exported by the
 * client to the server.
 *
 * GameObserver is a separate entity since GameView cannot
 * also be a Remote object.
 *
 * @author $Author: oscar $
 * @version $Id: GameObserver.java,v 1.1 2005/03/03 14:18:13 oscar Exp $
 **/
public class GameObserver
	extends UnicastRemoteObject
	implements RemoteObserver
{
	private GameGUI gui_;

	/**
	 * Do not use this constructor.
	 * Must be declared since we inherit from UnicastRemoteObject.
	 */
	public GameObserver() throws RemoteException {
		super();
	}

	/**
	 * This is the constructor to use.
	 */
	public GameObserver(GameGUI gui) throws RemoteException {
		gui_ = gui;
	}	

	/**
	 * Implements the RemoteObserver interface.
	 * Called by the BoardGame when its state changes.
	 *
	 * @param move an instance of Move
	 */
	public void update(Move move) throws RemoteException {
		gui_.update(move);
	}
}
