package p2.tictactoe.server;

import java.net.MalformedURLException;
import java.rmi.*;
import java.rmi.server.*;

import p2.tictactoe.*;
import p2.tictactoe.rmi.*;

/**
 * Implements factory method to instantiate BoardGame
 * for remote clients.
 * @author $Author: oscar $
 * @version $Id: GameServer.java,v 1.2 2005/03/03 16:00:52 oscar Exp $
 */
public class GameServer
extends UnicastRemoteObject
implements RemoteGameServer
{
	private GameFactory tictactoeFactory_, gomokuFactory_;

	/**
	 * main() method to create one instance of GameFactory
	 * and register it with the Naming service.
	 * @param args server:port (e.g., asterix.unibe.ch:2001)
	 * @throws RemoteException
	 * @throws MalformedURLException
	 **/
	public static void main(String[] args)
		throws RemoteException, MalformedURLException
	{
		if (System.getSecurityManager() == null) {
			System.setSecurityManager(new RMISecurityManager());
			System.out.println("Set new Security manager");
		} else {
			System.out.println("Security manager already set");
		}
		
		if (args.length != 1) {
			System.err.println("Usage: java "
					+ "p2.tictactoe.server.GameServer"
					+ " <RMI registry host:port address>");
			System.exit(1);
		}
		
		String name = "//" + args[0] + "/GameServer";
		
		// throws RemoteException
		RemoteGameServer server = new GameServer();
		System.out.println("Binding " + name + " ...");
		// thows MalformedURLException
		Naming.rebind(name, server);
		System.out.println("New GameServer registered");
	}

	public GameServer() throws RemoteException {
		super();
		tictactoeFactory_ = new TicTacToeFactory();
		gomokuFactory_ = new GomokuFactory();
	}
	
	/**
	 * Returns a RemoteGame instance for a remote client.
	 * If a game already exists, the client joins the
	 * existing game.  Else a new game is made.
	 *<P>
	 * This method is synchronized since multiple clients
	 * may try to connect concurrently.
	 */
	public synchronized RemoteGame joinTicTacToe() throws RemoteException {
		return new GameProxy(tictactoeFactory_.getGame());
	}
	
	public synchronized RemoteGame joinGomoku() throws RemoteException {
		return new GameProxy(gomokuFactory_.getGame());
	}

	private Player newPlayer(char mark) {
		return new PassivePlayer(mark);
	}
}
